unit GProperties;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls, GraphicEx, ComCtrls, Buttons, ExtCtrls;

type
  TGProperty = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    Label11: TLabel;
    TiledCheckBox: TCheckBox;
    EndianCheckBox: TCheckBox;
    MinCheckBox: TCheckBox;
    ReversedCheckBox: TCheckBox;
    InterlacedCheckBox: TCheckBox;
    Label7: TLabel;
    CompressionComboBox: TComboBox;
    Label1: TLabel;
    WidthEdit: TEdit;
    HeightEdit: TEdit;
    AOKPanel: TPanel;
    InfoBtn: TSpeedButton;
    HelpBtn: TSpeedButton;
    OKButton: TButton;
    Label10: TLabel;
    Label3: TLabel;
    ColorComboBox: TComboBox;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    BPPLabel: TLabel;
    SPPLabel: TLabel;
    BPSLabel: TLabel;
    DescriptionLabel: TLabel;
    Label9: TLabel;
    GammaCheckBox: TCheckBox;
    FileGammaLabel: TLabel;
    Label8: TLabel;
    DisplayGammaLabel: TLabel;
    FileGammaEdit: TEdit;
    Label2: TLabel;
    YResEdit: TEdit;
    XResEdit: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);


    procedure InfoBtnClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
  public
    Graphic: TGraphic;
  end;

var
  GProperty: TGProperty;

//----------------------------------------------------------------------------------------------------------------------

implementation

uses
  Globals,GraphicColor, GCfrm;

{$R *.DFM}

//----------------------------------------------------------------------------------------------------------------------
procedure TGProperty.FormCreate(Sender: TObject);
begin
  top := GPropertyDialogY;
  left := GPropertyDialogX;
end;

procedure TGProperty.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Graphic.free;
  GPropertyDialogX := GProperty.left;
  GPropertyDialogY := GProperty.top;
  DoSaver;
end;

procedure TGProperty.InfoBtnClick(Sender: TObject);
begin
{}
end;

procedure TGProperty.HelpBtnClick(Sender: TObject);
begin
{}
end;

procedure TGProperty.OKButtonClick(Sender: TObject);
begin
  ModalResult:=mrOk;
  Close;
end;


procedure TGProperty.FormShow(Sender: TObject);
var
  ImageProperties: TImageProperties;
begin
  OKButton.SetFocus;
  ZeroMemory(@ImageProperties, SizeOf(ImageProperties));
  with ImageProperties do
  begin
    if Graphic is TGraphicExGraphic then
    begin
      Caption :='Image properties'+ FileFormatList.GetDescription(TGraphicClass(Graphic.ClassType));
      ImageProperties := TGraphicExGraphic(Graphic).ImageProperties;
    end
    else
    begin
      Caption := 'Image properties unknown';
      // Delphi built-in classes,
      //try to get out at least some information
      Width := Graphic.Width;
      Height := Graphic.Height;
      if Graphic is TBitmap then
      begin
        case TBitmap(Graphic).PixelFormat of
          pf1Bit:
           begin
             BitsPerSample := 1;
             SamplesPerPixel := 1;
             ColorScheme := csIndexed;
           end;
          pf4Bit:
           begin
             BitsPerSample := 4;
             SamplesPerPixel := 1;
             ColorScheme := csIndexed;
           end;
          pf8Bit:
           begin
             BitsPerSample := 8;
             SamplesPerPixel := 1;
             ColorScheme := csIndexed;
           end;
          pf15Bit, pf16Bit:
           begin
             BitsPerSample := 5;
             SamplesPerPixel := 3;
             ColorScheme := csBGR;
           end;
          pf24Bit:
           begin
             BitsPerSample := 8;
             SamplesPerPixel := 3;
             ColorScheme := csBGR;
           end;
          pf32Bit:
           begin
             BitsPerSample := 8;
             SamplesPerPixel := 4;
             ColorScheme := csBGRA;
           end;
          pfDevice:
           begin
             BitsPerPixel := GetDeviceCaps(Canvas.Handle, BITSPIXEL);
             SamplesPerPixel := 8;
             BitsPerSample := BitsPerPixel div SamplesPerPixel;
             ColorScheme := csBGRA;
           end;
        end;{Case}
        BitsPerPixel := BitsPerSample * SamplesPerPixel;
      end;
    end; {Graphic type}

    // options
    TiledCheckBox.Checked := ioTiled in Options;
    EndianCheckBox.Checked := ioBigEndian in Options;
    MinCheckBox.Checked := ioMinIsWhite in Options;
    ReversedCheckBox.Checked := ioReversed in Options;
    InterlacedCheckBox.Checked := Interlaced;
    GammaCheckBox.Checked := ioUseGamma in Options;

    // colors
    if ColorScheme = csUnknown then ColorComboBox.ItemIndex := -1
                               else ColorComboBox.ItemIndex := Ord(ColorScheme) - 1;
    BPSLabel.Caption := IntToStr(BitsPerSample);
    SPPLabel.Caption := IntToStr(SamplesPerPixel);
    BPPLabel.Caption := IntToStr(BitsPerPixel);
    case ColorScheme of
      csG, csGA:
        DescriptionLabel.Caption := Format('%d gray levels', [1 shl BitsPerPixel]);
      csRGB, csRGBA, csBGR, csBGRA, csCMY, csCMYK, csCIELab, csYCbCr, csPhotoYCC:
        DescriptionLabel.Caption := Format('%d bit true color', [BitsPerPixel]);
    else
      // indexed colors
      if BitsPerPixel = 1 then DescriptionLabel.Caption := 'black and white'
                          else DescriptionLabel.Caption := Format('%d palette colors', [1 shl BitsPerPixel]);
    end;{Case}

    // compression
    if Compression = ctUnknown then CompressionComboBox.ItemIndex := -1
                               else CompressionComboBox.ItemIndex := Ord(Compression) - 1;

    // size
    WidthEdit.Text := Inttostr(Width);
    HeightEdit.Text := Inttostr(Height);

    // resolution
    XResEdit.Text := Inttostr(Round(XResolution));
    YResEdit.Text := Inttostr(Round(YResolution));

    // gamma
    DisplayGammaLabel.Caption := FloatToStr(DefaultDisplayGamma);
    FileGammaEdit.Text := FloatToStrF(FileGamma, ffFixed, 5, 2);
  end;{Image Properties}
end;
//--------------------------------------------------------
//--------------------------------------------------------








end.

