unit ImageCorrelateFrm;
//  Compare two 24-bit bitmaps and write statistical results to a .DAT file.
//  efg, April 1998.
// kludged to this dec 2004, ilh
interface

uses
    Windows,             // TRGBTriple
    SysUtils,            // Exception
    Graphics,            // TBitmap
    StatisticsLibrary,   // TDescriptiveStatistics
  jpg, jpeg, PNGImage,   JPEG_IO,
  Messages, Classes, Controls, Forms, Dialogs,
  Buttons, ExtCtrls, StdCtrls, ComCtrls;

  CONST
    PixelMaxCount = 65536;

  TYPE
    TRGBTripleArray = ARRAY[0..PixelMaxCount-1] OF TRGBTriple;
    pRGBArray = ^TRGBTripleArray;

    EFatalError = CLASS(Exception);

type
  TImageCorrelateForm = class(TForm)
    OpenDialog1: TOpenDialog;
    Panel1: TPanel;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    RichEdit1: TRichEdit;
    Splitter1: TSplitter;
    RichEdit2: TRichEdit;
    ProgressBar1: TProgressBar;
    SpeedButton4: TSpeedButton;
    HelpButton: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure SpeedButton4Click(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
  private
    { Private declarations }
    Bitmap1                :  TBitmap;
    Bitmap2                :  TBitmap;
    CorrelationBlue        :  TLinearRegression;
    CorrelationGreen       :  TLinearRegression;
    CorrelationRed         :  TLinearRegression;
    CorrelationIntensity   :  TLinearRegression;
    CorrelationFile        :  TextFile;
    DeltaBlue              :  DOUBLE;
    DeltaGreen             :  DOUBLE;
    DeltaRed               :  DOUBLE;
    DeltaDistance          :  DOUBLE;
    ParamStrID             :  INTEGER;
    ImageFile1,ImageFile2,
    OutFileName            :  STRING;
    row1                   :  pRGBArray;
    row2                   :  pRGBArray;
    StatisticsBlueAbsolute :  TDescriptiveStatistics;
    StatisticsBlueRelative :  TDescriptiveStatistics;
    StatisticsFile         :  TextFile;
    StatisticsGreenAbsolute:  TDescriptiveStatistics;
    StatisticsGreenRelative:  TDescriptiveStatistics;
    StatisticsRedAbsolute  :  TDescriptiveStatistics;
    StatisticsRedRelative  :  TDescriptiveStatistics;
    StatisticsDistance     :  TDescriptiveStatistics;
  public
    { Public declarations }
  end;

var
  ImageCorrelateForm: TImageCorrelateForm;

implementation

{$R *.DFM}
uses  TongGlobals;

procedure TImageCorrelateForm.FormCreate(Sender: TObject);
begin
  ParamStrID:=0;
  left := ImageCorrelateFormX;
  top := ImageCorrelateFormY;
end;

procedure TImageCorrelateForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  ImageCorrelateFormX := ImageCorrelateForm.left;
  ImageCorrelateFormY := ImageCorrelateForm.top;
end;
procedure TImageCorrelateForm.HelpButtonClick(Sender: TObject);
begin
  Application.HelpContext(6555);
end;
procedure TImageCorrelateForm.SpeedButton1Click(Sender: TObject);
begin
    OpenDialog1.InitialDir := ImagePath;
    OpenDialog1.Filter := 'All (*.bmp;*.png)|*.bmp;*.png;*.jpg';
  if OpenDialog1.Execute then          { Display Open dialog box }
     ImageFile1:=OpenDialog1.FileName;
end;

procedure TImageCorrelateForm.SpeedButton2Click(Sender: TObject);
begin
    OpenDialog1.InitialDir := ImagePath;
    OpenDialog1.Filter := 'All (*.bmp;*.png)|*.bmp;*.png;*.jpg';
  if OpenDialog1.Execute then          { Display Open dialog box }
     ImageFile2:=OpenDialog1.FileName;
end;

procedure TImageCorrelateForm.SpeedButton3Click(Sender: TObject);
var i,j:Integer;
  PNG:TPNGObject;
  jp: TJpegImage;
  F: file;
begin
RichEdit1.Clear;
RichEdit2.Clear;
ProgressBar1.Position:=10;
    IF   NOT FileExists(ImageFile1)
    THEN RAISE EFatalError.Create('File 1 cannot be found "' + ParamStr(1) + '"');
    Bitmap1 := TBitmap.Create;
  if (lowercase(extractfileext(ImageFile1)) = '.bmp') then begin
    Bitmap1.LoadFromFile(ImageFile1);
  end else
  if (lowercase(extractfileext(ImageFile1)) = '.jpg') then
  begin
        If IntelJpgAvailable then
        begin
        LoadBmpFromJpegFile (Bitmap1,
           ImageFile1, True, ijlFullSize);
        end else
        begin
        jp := TJpegImage.Create;
        try
          with jp do
          begin
          jp.LoadFromFile(ImageFile1);
          Bitmap1.Assign(jp);
          end;
        finally
          jp.Free;
        end;
        end;
  end else
  if (lowercase(extractfileext(ImageFile1)) = '.png') then
  begin
    PNG := TPNGObject.Create;
    {In case something goes wrong, free both PNG and Bitmap}
    try
      PNG.LoadFromFile(ImageFile1);
      Bitmap1.Assign(PNG); //Convert data into bitmap
      finally
      PNG.Free;
    end;
  end;

    IF   NOT FileExists(ImageFile2)
    THEN RAISE EFatalError.Create('File 2 cannot be found "' + ParamStr(2) + '"');
    Bitmap2 := TBitmap.Create;
  if (lowercase(extractfileext(ImageFile2)) = '.bmp') then begin
    Bitmap2.LoadFromFile(ImageFile2);
  end else
  if (lowercase(extractfileext(ImageFile2)) = '.jpg') then
  begin
        If IntelJpgAvailable then
        begin
        LoadBmpFromJpegFile (Bitmap2,
           ImageFile2, True, ijlFullSize);
        end else
        begin
        jp := TJpegImage.Create;
        try
          with jp do
          begin
          jp.LoadFromFile(ImageFile2);
          Bitmap2.Assign(jp);
          end;
        finally
          jp.Free;
        end;
        end;
  end else
  if (lowercase(extractfileext(ImageFile2)) = '.png') then
  begin
    PNG := TPNGObject.Create;
    {In case something goes wrong, free both PNG and Bitmap}
    try
      PNG.LoadFromFile(ImageFile2);
      Bitmap2.Assign(PNG); //Convert data into bitmap
      finally
      PNG.Free;
    end;
  end;

    IF  (Bitmap1.Width  <> Bitmap2.Width)  OR
        (Bitmap1.Height <> Bitmap2.Height)
    THEN RAISE EFatalError.Create('Bitmaps do not have the same dimensions:  ' +
                                  IntToStr(Bitmap1.Width) + '-by-' + IntToStr(Bitmap1.Height) + ' versus ' +
                                  IntToStr(Bitmap2.Width) + '-by-' + IntToStr(Bitmap2.Height) );


    IF  (Bitmap1.PixelFormat <> pf24Bit) OR
        (Bitmap2.PixelFormat <> pf24Bit)
    THEN RAISE EFatalError.Create('Only 24-bit bitmaps supported');

    inc(ParamStrID);
    OutFilename := 'Statistics.Dat';
    AssignFile(StatisticsFile, OutFilename);
    AssignFile(CorrelationFile, 'Correlation.DAT');

    IF   FileExists(OutFilename)
    THEN BEGIN
      Append(StatisticsFile);
      Append(CorrelationFile)
    END
    ELSE BEGIN
      REWRITE(StatisticsFile);

      WRITE(StatisticsFile,   '          ');
      WRITE(StatisticsFile,   '<----------------- Red -----------------> ');
      WRITE(StatisticsFile,   '<---------------- Green ----------------> ');
      WRITE(StatisticsFile,   '<----------------- Blue ----------------> ');
      WRITELN(StatisticsFile, '<----- RGB Composite ----->');

      WRITE(StatisticsFile,   '          ');
      WRITE(StatisticsFile,   '    Absolute      Relative                ');
      WRITE(StatisticsFile,   '    Absolute      Relative                ');
      WRITE(StatisticsFile,   '    Absolute      Relative                ');
      WRITELN(StatisticsFile, '    Absolute');


      WRITE(StatisticsFile,   '   ID     ');
      FOR i := 1 TO 3 DO
      BEGIN
        WRITE(StatisticsFile, Format(' %5s  %5s  %5s  %5s  %5s %5s |',
              [' mean', ' S.D.', ' mean', ' S.D.', ' min ', ' max ']))
      END;
      WRITELN(StatisticsFile,
            Format(' %5s  %5s  %5s  %5s', [' mean', ' S.D.', ' min ', ' max ']));

      WRITE(StatisticsFile,   '--------  ');
      FOR i := 1 TO 3 DO
      BEGIN
        WRITE(StatisticsFile, Format(' %5s  %5s  %5s  %5s  %5s %5s |',
              ['-----', '-----', '-----', '-----', '-----', '-----']))
      END;
      WRITELN(StatisticsFile,
            Format(' %5s  %5s  %5s  %5s', ['-----', '-----', '-----', '-----']));


      REWRITE(CorrelationFile);
      WRITELN(CorrelationFile,  '   ID       Red     Green    Blue      RGB');
      WRITELN(CorrelationFile,  '--------  -------  -------  -------  -------')
    END;
    Application.ProcessMessages;
    StatisticsRedAbsolute      := TDescriptiveStatistics.Create;
    StatisticsGreenAbsolute    := TDescriptiveStatistics.Create;
    StatisticsBlueAbsolute     := TDescriptiveStatistics.Create;

    StatisticsRedRelative      := TDescriptiveStatistics.Create;
    StatisticsGreenRelative    := TDescriptiveStatistics.Create;
    StatisticsBlueRelative     := TDescriptiveStatistics.Create;

    StatisticsDistance := TDescriptiveStatistics.Create;

    CorrelationRed       := TLinearRegression.Create;
    CorrelationGreen     := TLinearRegression.Create;
    CorrelationBlue      := TLinearRegression.Create;
    CorrelationIntensity := TLinearRegression.Create;

    TRY

      FOR j := 0 TO Bitmap1.Height-1 DO
      BEGIN
        row1       := Bitmap1.ScanLine[j];
        row2       := Bitmap2.ScanLine[j];
        ProgressBar1.Position:= round(j/Bitmap1.Height);
        Application.ProcessMessages;
        FOR i := 0 TO Bitmap1.Width-1 DO
        BEGIN
          CorrelationRed.NextValue(row1[i].rgbtRed,  row2[i].rgbtRed);
          CorrelationGreen.NextValue(row1[i].rgbtGreen,  row2[i].rgbtGreen);
          CorrelationBlue.NextValue(row1[i].rgbtBlue,  row2[i].rgbtBlue);
          CorrelationIntensity.NextValue((row1[i].rgbtRed + row1[i].rgbtGreen + row1[i].rgbtBlue) / 3,
                                         (row2[i].rgbtRed + row2[i].rgbtGreen + row2[i].rgbtBlue) / 3);

          DeltaRed      := INTEGER(row1[i].rgbtRed)   - INTEGER(row2[i].rgbtRed);
          DeltaGreen    := INTEGER(row1[i].rgbtGreen) - INTEGER(row2[i].rgbtGreen);
          DeltaBlue     := INTEGER(row1[i].rgbtBlue)  - INTEGER(row2[i].rgbtBlue);
          DeltaDistance := SQRT( SQR(DeltaRed) + SQR(DeltaGreen) + SQR(DeltaBlue) );

          StatisticsRedRelative.NextValue(DeltaRed);           // -255 .. 255
          StatisticsGreenRelative.NextValue(DeltaGreen);       // -255 .. 255
          StatisticsBlueRelative.NextValue(DeltaBlue);         // -255 .. 255

          StatisticsRedAbsolute.NextValue(ABS(DeltaRed));      // 0 .. 255
          StatisticsGreenAbsolute.NextValue(ABS(DeltaGreen));  // 0 .. 255
          StatisticsBlueAbsolute.NextValue(ABS(DeltaBlue));    // 0 .. 255

          StatisticsDistance.NextValue(DeltaDistance);         // 0 .. SQRT(255^3)

        END

      END;
      Application.ProcessMessages;
      WRITE(StatisticsFile, ParamStrID:8, '  ');   // "ID" field



      WRITE(StatisticsFile,

            Format(' %5.1f  %5.1f  %5.1f  %5.1f  %5.0f %5.0f |',
                   [StatisticsRedAbsolute.MeanValue,
                    StatisticsRedAbsolute.StandardDeviation,
                    StatisticsRedRelative.MeanValue,
                    StatisticsRedRelative.StandardDeviation,
                    StatisticsRedRelative.MinValue,
                    StatisticsRedRelative.Maxvalue]));

      WRITE(StatisticsFile,
            Format(' %5.1f  %5.1f  %5.1f  %5.1f  %5.0f %5.0f |',
                   [StatisticsGreenAbsolute.MeanValue,
                    StatisticsGreenAbsolute.StandardDeviation,
                    StatisticsGreenRelative.MeanValue,
                    StatisticsGreenRelative.StandardDeviation,
                    StatisticsGreenRelative.MinValue,
                    StatisticsGreenRelative.Maxvalue]));

      WRITE(StatisticsFile,
            Format(' %5.1f  %5.1f  %5.1f  %5.1f  %5.0f %5.0f |',
                   [StatisticsBlueAbsolute.MeanValue,
                    StatisticsBlueAbsolute.StandardDeviation,
                    StatisticsBlueRelative.MeanValue,
                    StatisticsBlueRelative.StandardDeviation,
                    StatisticsBlueRelative.MinValue,
                    StatisticsBlueRelative.Maxvalue]));

      WRITELN(StatisticsFile,
            Format(' %5.1f  %5.1f  %5.0f  %5.0f',
                   [StatisticsDistance.MeanValue,
                    StatisticsDistance.StandardDeviation,
                    StatisticsDistance.MinValue,
                    StatisticsDistance.Maxvalue]));

      WRITELN(CorrelationFile,
              Format('%8s %8.5f %8.5f %8.5f %8.5f',
                     [ParamStr(3), CorrelationRed.CorrelationCoefficient,
                                   CorrelationGreen.CorrelationCoefficient,
                                   CorrelationBlue.CorrelationCoefficient,
                                   CorrelationIntensity.CorrelationCoefficient]));

    FINALLY
      StatisticsRedAbsolute.Free;
      StatisticsGreenAbsolute.Free;
      StatisticsBlueAbsolute.Free;

      StatisticsRedRelative.Free;
      StatisticsGreenRelative.Free;
      StatisticsBlueRelative.Free;

      StatisticsDistance.Free;

      CorrelationRed.Free;
      CorrelationGreen.Free;
      CorrelationBlue.Free;
      CorrelationIntensity.Free;

      Bitmap1.Free;
      Bitmap2.Free;

      CloseFile(StatisticsFile);
      CloseFile(CorrelationFile)
    END;
ProgressBar1.Position:=50;
Application.ProcessMessages;
RichEdit1.Lines.Loadfromfile('Statistics.Dat');
RichEdit2.Lines.Loadfromfile('Correlation.DAT');
ProgressBar1.Position:=0;
Application.ProcessMessages;
AssignFile(F, 'Statistics.Dat');
Reset(F);CloseFile(F); Erase(F);
Application.ProcessMessages;
AssignFile(F, 'Correlation.DAT');
Reset(F);CloseFile(F); Erase(F);
end;

procedure TImageCorrelateForm.SpeedButton4Click(Sender: TObject);
begin
  RichEdit1.Lines.Savetofile('Statistics.Dat');
  RichEdit2.Lines.Savetofile('Correlation.DAT');
end;



end.
