unit Nelson2DGeometry;

interface

type
  TVector = array[0..1] of single;

procedure VectorScale(V: array of single; Factor: single);
//procedure VectorScale(var V: array of single; Factor: single);
function  VectorSubtract(V1, V2: TVector): TVector;
function  VectorLength(V: array of single): single;
//function  VectorLength(V: TVector): single;
function  VectorCombine(V1, V2: TVector; F1, F2: single): TVector;
function  VectorAdd(V1, V2: TVector): TVector;
function  VectorDotProduct(V1, V2: TVector): single;

implementation

const
  X = 0;
  Y = 1;

procedure VectorScale(V: array of single; Factor: single); assembler; register;

// returns a vector scaled by a factor
// EAX contains address of V
// EDX contains highest index in V
// Factor is located on the stack
// for I := Low(V) to High(V) do V[I] := V[I] * Factor;
asm


              FLD DWORD PTR [Factor]        // load factor
@@Loop:       FLD DWORD PTR [EAX + 4 * EDX] // load a component
              FMUL ST, ST(1)                // multiply it with the factor
              WAIT
              FSTP DWORD PTR [EAX + 4 * EDX] // store the result
              DEC EDX                       // do the entire array
              JNS @@Loop
              FSTP ST(0)                    // clean the FPU stack
end;

{procedure VectorScale(var V: array of single; Factor: single);
var i:integer;
begin
  for I := Low(V) to High(V) do V[I] := V[I] * Factor;
end;}

function  VectorSubtract(V1, V2: TVector): TVector; register;
begin
  Result[X] := V1[X] - V2[X];
  Result[Y] := V1[Y] - V2[Y];
end;

{function VectorLength(V: TVector): single;
begin
  result:= sqrt( V[0]*V[0] + V[1]*V[1] );
end;}

function VectorLength(V: array of single): single; assembler;

// calculates the length of a vector following the equation: sqrt(x * x + y * y + ...)
// Note: The parameter of this function is declared as open array. Thus
// there's no restriction about the number of the components of the vector.
//
// EAX contains address of V
// EDX contains the highest index of V
// the result is returned in ST(0)

asm
              FLDZ                           // initialize sum
@@Loop:       FLD  DWORD PTR [EAX  +  4 * EDX] // load a component
              FMUL ST, ST
              FADDP
              SUB  EDX, 1
              JNL  @@Loop
              FSQRT
end;

function  VectorCombine(V1, V2: TVector; F1, F2: single): TVector;
begin
  Result[X] := (F1 * V1[X]) + (F2 * V2[X]);
  Result[Y] := (F1 * V1[Y]) + (F2 * V2[Y]);
end;

function  VectorAdd(V1, V2: TVector): TVector; register;
begin
  Result[X] := V1[X] + V2[X];
  Result[Y] := V1[Y] + V2[Y];
end;

function  VectorDotProduct(V1, V2: TVector): single;
begin
  Result := V1[X] * V2[X] + V1[Y] * V2[Y];
end;



end.
