{#####################################################################################}
{##                                                                                 ##}
{## Texture_3D_Unit                                                                 ##}
{##                                                                                 ##}
{## A small sample class for 3D textures                                            ##}
{##                                                                                 ##}
{## History                                                                         ##}
{##   13.12.2003 : Jrgen Abel  : First version                                     ##}
{##                                                                                 ##}
{#####################################################################################}


Unit Texture_3D_Unit;


Interface


Uses
  Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, GLScene, GLObjects, GLWin32Viewer, GLMisc, GLTexture, GLContext,
  OpenGL1x, VectorGeometry, GLCadencer, ExtCtrls, ComCtrls, StdCtrls;


Type
  TTexture_3D = Class (TObject)
  Protected
    M_Data_Type : Longword;
    M_X_Size : Integer;
    M_Y_Size : Integer;
    M_Z_Size : Integer;
    M_Texel_Byte_Size : Integer;
    M_Data : String;
    Function Data_Type_To_Texel_Byte_Size (F_Data_Type : Integer) : Integer;
    Procedure Set_Data_Type (F_Value : Longword);
    Procedure Set_X_Size (F_Value : Integer);
    Procedure Set_Y_Size (F_Value : Integer);
    Procedure Set_Z_Size (F_Value : Integer);
    Procedure Set_Data (Const F_Value : String);
  Public
    Constructor Create;
    Destructor Destroy; Override;
    Procedure Save_To_File (Const F_FileName : String);
    Procedure Load_From_File (Const F_FileName : String);
  Published
    Property Data_Type : Longword Read M_Data_Type Write Set_Data_Type;
    Property X_Size : Integer Read M_X_Size Write Set_X_Size;
    Property Y_Size : Integer Read M_Y_Size Write Set_Y_Size;
    Property Z_Size : Integer Read M_Z_Size Write Set_Z_Size;
    Property Texel_Byte_Size : Integer Read M_Texel_Byte_Size;
    Property Data : String Read M_Data Write Set_Data;
  End; { TTexture_3D }


Implementation


{-------------------------------------------------------------------------------------}
Constructor TTexture_3D.Create;
{ Initializes dynamical data                                                          }
{-------------------------------------------------------------------------------------}
Begin { TTexture_3D.Create }
  Inherited Create;

  { Initialize variables }
  M_Data_Type := GL_RGBA;
  M_X_Size := 0;
  M_Y_Size := 0;
  M_Z_Size := 0;
  M_Texel_Byte_Size := Data_Type_To_Texel_Byte_Size (M_Data_Type);
  SetLength (M_Data, M_X_Size * M_Y_Size * M_Z_Size * M_Texel_Byte_Size);
End; { TTexture_3D.Create }


{-------------------------------------------------------------------------------------}
Destructor TTexture_3D.Destroy;
{ Release dynamic data                                                                }
{-------------------------------------------------------------------------------------}
Begin { TTexture_3D.Destroy }
  { Free data }
  SetLength (M_Data, 0);

  Inherited Destroy;
End; { TTexture_3D.Destroy }


{-------------------------------------------------------------------------------------}
Function TTexture_3D.Data_Type_To_Texel_Byte_Size (F_Data_Type : Integer) : Integer;
{ Calculates texel size in bytes                                                      }
{-------------------------------------------------------------------------------------}
Begin { TTexture_3D.Data_Type_To_Texel_Byte_Size }
  Case F_Data_Type Of
    GL_COLOR_INDEX :
      Begin { GL_COLOR_INDEX }
        Result := 1;
      End; { GL_COLOR_INDEX }
    GL_STENCIL_INDEX :
      Begin { GL_STENCIL_INDEX }
        Result := 1;
      End; { GL_STENCIL_INDEX }
    GL_DEPTH_COMPONENT :
      Begin { GL_DEPTH_COMPONENT }
        Result := 1;
      End; { GL_DEPTH_COMPONENT }
    GL_RED :
      Begin { GL_RED }
        Result := 1;
      End; { GL_RED }
    GL_GREEN :
      Begin { GL_GREEN }
        Result := 1;
      End; { GL_GREEN }
    GL_BLUE :
      Begin { GL_BLUE }
        Result := 1;
      End; { GL_BLUE }
    GL_ALPHA :
      Begin { GL_ALPHA }
        Result := 1;
      End; { GL_ALPHA }
    GL_RGB :
      Begin { GL_RGB }
        Result := 3;
      End; { GL_RGB }
    GL_RGBA :
      Begin { GL_RGBA }
        Result := 4;
      End; { GL_RGBA }
    GL_LUMINANCE :
      Begin { GL_LUMINANCE }
        Result := 1;
      End; { GL_LUMINANCE }
    GL_LUMINANCE_ALPHA :
      Begin { GL_LUMINANCE_ALPHA }
        Result := 2;
      End; { GL_LUMINANCE_ALPHA }
    Else
      Begin { else }
        Result := 4;
      End; { else }
  End; { Case }
End; { TTexture_3D.Data_Type_To_Texel_Byte_Size }


{-------------------------------------------------------------------------------------}
Procedure TTexture_3D.Set_Data_Type (F_Value : Longword);
{ Sets data type                                                                      }
{-------------------------------------------------------------------------------------}
Begin { TTexture_3D.Set_Data_Type }
  M_Data_Type := F_Value;

  M_Texel_Byte_Size := Data_Type_To_Texel_Byte_Size (M_Data_Type);
  SetLength (M_Data, M_X_Size * M_Y_Size * M_Z_Size * M_Texel_Byte_Size);
End; { TTexture_3D.Set_Data_Type }


{-------------------------------------------------------------------------------------}
Procedure TTexture_3D.Set_X_Size (F_Value : Integer);
{ Sets X size                                                                         }
{-------------------------------------------------------------------------------------}
Begin { TTexture_3D.Set_X_Size }
  M_X_Size := F_Value;

  SetLength (M_Data, M_X_Size * M_Y_Size * M_Z_Size * M_Texel_Byte_Size);
End; { TTexture_3D.Set_X_Size }


{-------------------------------------------------------------------------------------}
Procedure TTexture_3D.Set_Y_Size (F_Value : Integer);
{ Sets Y size                                                                         }
{-------------------------------------------------------------------------------------}
Begin { TTexture_3D.Set_Y_Size }
  M_Y_Size := F_Value;

  SetLength (M_Data, M_X_Size * M_Y_Size * M_Z_Size * M_Texel_Byte_Size);
End; { TTexture_3D.Set_Y_Size }


{-------------------------------------------------------------------------------------}
Procedure TTexture_3D.Set_Z_Size (F_Value : Integer);
{ Sets Z size                                                                         }
{-------------------------------------------------------------------------------------}
Begin { TTexture_3D.Set_Z_Size }
  M_Z_Size := F_Value;

  SetLength (M_Data, M_X_Size * M_Y_Size * M_Z_Size * M_Texel_Byte_Size);
End; { TTexture_3D.Set_Z_Size }


{-------------------------------------------------------------------------------------}
Procedure TTexture_3D.Set_Data (Const F_Value : String);
{ Sets data                                                                           }
{-------------------------------------------------------------------------------------}
Begin { TTexture_3D.Set_Data }
  SetLength (M_Data, Length (F_Value));
  M_Data := F_Value;
End; { TTexture_3D.Set_Data }


{-------------------------------------------------------------------------------------}
Procedure TTexture_3D.Save_To_File (Const F_FileName : String);
{ Save data to file                                                                   }
{-------------------------------------------------------------------------------------}
Var
  File_Stream : TFileStream;

Begin { TTexture_3D.Save_To_File }
  File_Stream := TFileStream.Create (F_FileName, fmCreate or fmShareDenyWrite);
  Try
    File_Stream.WriteBuffer (M_Data_Type, SizeOf (Longword));
    File_Stream.WriteBuffer (M_X_Size, SizeOf (Integer));
    File_Stream.WriteBuffer (M_Y_Size, SizeOf (Integer));
    File_Stream.WriteBuffer (M_Z_Size, SizeOf (Integer));
    File_Stream.WriteBuffer (PChar (M_Data)^, Length (M_Data));
  Finally
    File_Stream.Free;
  End; { Try }
End; { TTexture_3D.Save_To_File }


{-------------------------------------------------------------------------------------}
Procedure TTexture_3D.Load_From_File (Const F_FileName : String);
{ Load data from file                                                                 }
{-------------------------------------------------------------------------------------}
Var
  File_Stream : TFileStream;

Begin { TTexture_3D.Load_From_File }
  File_Stream := TFileStream.Create (F_FileName, fmOpenRead or fmShareDenyWrite);
  Try
    File_Stream.ReadBuffer (M_Data_Type, SizeOf (Longword));
    File_Stream.ReadBuffer (M_X_Size, SizeOf (Integer));
    File_Stream.ReadBuffer (M_Y_Size, SizeOf (Integer));
    File_Stream.ReadBuffer (M_Z_Size, SizeOf (Integer));
    M_Texel_Byte_Size := Data_Type_To_Texel_Byte_Size (M_Data_Type);
    SetLength (M_Data, M_X_Size * M_Y_Size * M_Z_Size * M_Texel_Byte_Size);
    File_Stream.ReadBuffer (PChar (M_Data)^, Length (M_Data));
  Finally
    File_Stream.Free;
  End; { Try }
End; { TTexture_3D.Load_From_File }


End.

