{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit gOMark;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ColorGrd, ComCtrls, Buttons;

type
  TObjectMarkForm = class(TForm)
    LoadListBtn: TSpeedButton;
    SaveListBtn: TSpeedButton;
    HelpBtn: TSpeedButton;
    OverlayListBtn: TSpeedButton;
    UpDown1: TUpDown;
    CurrentListEdit: TEdit;
    Add1toListBtn: TSpeedButton;
    Delete1ListBtn: TSpeedButton;
    SolidColorGrid: TColorGrid;
    MousedLocationCB: TCheckBox;
    LOX1Edit: TEdit;
    LOX2Edit: TEdit;
    ObjectRG: TRadioGroup;
    FillCB: TCheckBox;
    LOY1Edit: TEdit;
    LOY2Edit: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    LineColorGrid: TColorGrid;
    LineColorLabel: TLabel;
    OMOpenDialog: TOpenDialog;
    OMSaveDialog1: TSaveDialog;
    LineColorPanel: TPanel;
    FillPanel: TPanel;
    PlaceItBtn: TSpeedButton;
    ClearAllBtn: TSpeedButton;
    MakeBtn: TSpeedButton;
    Label5: TLabel;
    ExitButton: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure HelpBtnClick(Sender: TObject);
    function GetColorGridIndex(Incolor: TColor): Integer;
    procedure LineColorGridClick(Sender: TObject);
    procedure SolidColorGridClick(Sender: TObject);
    procedure UpDown1ChangingEx(Sender: TObject; var AllowChange:
      Boolean;
      NewValue: Smallint; Direction: TUpDownDirection);
    procedure UpDown1Click(Sender: TObject; Button: TUDBtnType);
    procedure SelectNextListItem(Rotate: Boolean);
    procedure Delete1ListBtnClick(Sender: TObject);
    procedure GetData(Which: Integer);
    procedure ShowData(Which: Integer);
    procedure Add1toListBtnClick(Sender: TObject);
    procedure PlaceItBtnClick(Sender: TObject);
    procedure LoadListBtnClick(Sender: TObject);
    procedure SaveListBtnClick(Sender: TObject);
    procedure OverlayListBtnClick(Sender: TObject);
    procedure PaintAll;
    procedure ClearAllBtnClick(Sender: TObject);
    procedure MakeBtnClick(Sender: TObject);
    procedure ExitButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  ObjectMarkForm: TObjectMarkForm;
  CurrentObjectMarker,
    ObjectListCount: Integer;

implementation

uses gGlobal, gGrip;

{$R *.DFM}

procedure TObjectMarkForm.FormCreate(Sender: TObject);
begin
  left := ObjectMarkFormX;
  top := ObjectMarkFormY;
  ObjectListCount := 0;
  CurrentObjectMarker := 0;
  ObjectMarking := False;
  ObjectMarkingOn := False;
  SetLength(ObjectMarkerArray, ObjectListCount + 1);
end;

procedure TObjectMarkForm.ExitButtonClick(Sender: TObject);
begin
  Close;
end;

procedure TObjectMarkForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  ObjectMarkFormX := ObjectMarkForm.left;
  ObjectMarkFormY := ObjectMarkForm.top;
  DoSaver;
  FormControl:=11;  
  GRIPForm.FreeForm;   
end;

procedure TObjectMarkForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(430);
end;

function TObjectMarkForm.GetColorGridIndex(Incolor: TColor): Integer;
begin
  case Incolor of
    clBlack: Result := 0;
    clMaroon: Result := 1;
    clGreen: Result := 2;
    clOlive: Result := 3;
    clNavy: Result := 4;
    clPurple: Result := 5;
    clTeal: Result := 6;
    clGray: Result := 7;
    clSilver: Result := 8;
    clRed: Result := 9;
    clLime: Result := 10;
    clYellow: Result := 11;
    clBlue: Result := 12;
    clFuchsia: Result := 13;
    clAqua: Result := 14;
    clWhite: Result := 15;
  else Result := 15;
  end;
end;

procedure TObjectMarkForm.LineColorGridClick(Sender: TObject);
var ColorChoice: Integer;
begin
{Choose the Color and place choice into Shadow Edit box}
{clBlack,clMaroon,clGreen,clOlive,clNavy,clPurple,clTeal,
clGray,clSilver,clRed,clLime,clYellow,clBlue,clFuchsia,clAqua,clWhite
clDkGray,clLtGray,}
  ColorChoice := LineColorGrid.ForegroundIndex; {BackgroundIndex;}
  case ColorChoice of
    0: begin LineColorPanel.Color := clBlack; end;
    1: begin LineColorPanel.Color := clMaroon; end;
    2: begin LineColorPanel.Color := clGreen; end;
    3: begin LineColorPanel.Color := clOlive; end;
    4: begin LineColorPanel.Color := clNavy; end;
    5: begin LineColorPanel.Color := clPurple; end;
    6: begin LineColorPanel.Color := clTeal; end;
    7: begin LineColorPanel.Color := clGray; end;
    8: begin LineColorPanel.Color := clSilver; end;
    9: begin LineColorPanel.Color := clRed; end;
    10: begin LineColorPanel.Color := clLime; end;
    11: begin LineColorPanel.Color := clYellow; end;
    12: begin LineColorPanel.Color := clBlue; end;
    13: begin LineColorPanel.Color := clFuchsia; end;
    14: begin LineColorPanel.Color := clAqua; end;
    15: begin LineColorPanel.Color := clWhite; end;
  end;
end;


procedure TObjectMarkForm.SolidColorGridClick(Sender: TObject);
var ColorChoice: Integer;
begin
{Choose the Color and place choice into Shadow Edit box}
{clBlack,clMaroon,clGreen,clOlive,clNavy,clPurple,clTeal,
clGray,clSilver,clRed,clLime,clYellow,clBlue,clFuchsia,clAqua,clWhite
clDkGray,clLtGray,}
  ColorChoice := SolidColorGrid.ForegroundIndex; {BackgroundIndex;}
  case ColorChoice of
    0: begin FillPanel.Color := clBlack; end;
    1: begin FillPanel.Color := clMaroon; end;
    2: begin FillPanel.Color := clGreen; end;
    3: begin FillPanel.Color := clOlive; end;
    4: begin FillPanel.Color := clNavy; end;
    5: begin FillPanel.Color := clPurple; end;
    6: begin FillPanel.Color := clTeal; end;
    7: begin FillPanel.Color := clGray; end;
    8: begin FillPanel.Color := clSilver; end;
    9: begin FillPanel.Color := clRed; end;
    10: begin FillPanel.Color := clLime; end;
    11: begin FillPanel.Color := clYellow; end;
    12: begin FillPanel.Color := clBlue; end;
    13: begin FillPanel.Color := clFuchsia; end;
    14: begin FillPanel.Color := clAqua; end;
    15: begin FillPanel.Color := clWhite; end;
  end;
end;

procedure TObjectMarkForm.UpDown1ChangingEx(Sender: TObject;
  var AllowChange: Boolean; NewValue: Smallint;
  Direction: TUpDownDirection);
begin
  if (NewValue > ObjectListCount) then AllowChange := False
  else if (NewValue < 0) then AllowChange := False
  else AllowChange := True;
  if AllowChange then GetData(CurrentObjectMarker);
end;

procedure TObjectMarkForm.UpDown1Click(Sender: TObject;
  Button: TUDBtnType);
begin
  SelectNextListItem(Button = btNext);
end;

procedure TObjectMarkForm.SelectNextListItem(Rotate: Boolean);
begin
  if Rotate then inc(CurrentObjectMarker) else
    dec(CurrentObjectMarker);
  ShowData(CurrentObjectMarker);
end;

procedure TObjectMarkForm.ShowData(Which: Integer);
begin
  ObjectRG.ItemIndex := ObjectMarkerArray[Which].ObjectType;
  LOX1Edit.Text := IntToStr(ObjectMarkerArray[Which].X1);
  LOX2Edit.Text := IntToStr(ObjectMarkerArray[Which].X2);
  LOY1Edit.Text := IntToStr(ObjectMarkerArray[Which].Y1);
  LOY2Edit.Text := IntToStr(ObjectMarkerArray[Which].Y2);
  Label5.Caption := IntToStr(Which);
  LineColorPanel.Color := ObjectMarkerArray[Which].LineColor;
  LineColorGrid.ForeGroundIndex :=
    GetColorGridIndex(LineColorPanel.Color);
  FillPanel.Color := ObjectMarkerArray[Which].FillColor;
  SolidColorGrid.ForeGroundIndex :=
    GetColorGridIndex(FillPanel.Color);
  FillCB.Checked := ObjectMarkerArray[Which].IsSolidFill;
end;

procedure TObjectMarkForm.GetData(Which: Integer);
begin
  ObjectMarkerArray[Which].ObjectType := ObjectRG.ItemIndex;
  ObjectMarkerArray[Which].X1 := StrToInt(LOX1Edit.Text);
  ObjectMarkerArray[Which].X2 := StrToInt(LOX2Edit.Text);
  ObjectMarkerArray[Which].Y1 := StrToInt(LOY1Edit.Text);
  ObjectMarkerArray[Which].Y2 := StrToInt(LOY2Edit.Text);
  ObjectMarkerArray[Which].LineColor := LineColorPanel.Color;
  ObjectMarkerArray[Which].FillColor := FillPanel.Color;
  ObjectMarkerArray[Which].IsSolidFill := FillCB.Checked;
end;


procedure TObjectMarkForm.Delete1ListBtnClick(Sender: TObject);
var I: Integer;
begin
  if (ObjectListCount > 0) then
  begin
    if CurrentObjectMarker = ObjectListCount then
    begin
      dec(CurrentObjectMarker);
      dec(ObjectListCount);
      SetLength(ObjectMarkerArray, ObjectListCount + 1);
    end else
    begin
      for I := CurrentObjectMarker to ObjectListCount - 1 do
      begin
        ObjectMarkerArray[I] := ObjectMarkerArray[I + 1];
      end;
      CurrentObjectMarker := 0;
      dec(ObjectListCount);
      SetLength(ObjectMarkerArray, ObjectListCount + 1);
    end;
    ShowData(CurrentObjectMarker);
    UpDown1.Position := CurrentObjectMarker;
  end;
end;



procedure TObjectMarkForm.Add1toListBtnClick(Sender: TObject);
begin
  {Store current}
  GetData(CurrentObjectMarker);
  {Set next}
  inc(ObjectListCount);
  CurrentObjectMarker := ObjectListCount;
  SetLength(ObjectMarkerArray, ObjectListCount + 1);
  GetData(CurrentObjectMarker);
  ShowData(CurrentObjectMarker);
  UpDown1.Position := CurrentObjectMarker;
end;

procedure TObjectMarkForm.PlaceItBtnClick(Sender: TObject);
begin
  GetData(CurrentObjectMarker);
  if MousedLocationCB.Checked then
  begin
    ObjectMarking := True; {ObjectMarkingOn}
    GRIPForm.Show;
  end else
  begin
    ObjectMarkingOn := False;
    ObjectMarking := False;
    pointStart.x := StrToInt(LOX1Edit.Text);
    pointStart.y := StrToInt(LOY1Edit.Text);
    pointEnd.x := StrToInt(LOX2Edit.Text);
    pointEnd.y := StrToInt(LOY2Edit.Text);
    GRIPForm.Show;
    GRIPForm.DrawRubberMark;
  end;
end;

procedure TObjectMarkForm.LoadListBtnClick(Sender: TObject);
var fLoad: file of ObjectMarker;
  Incoming: ObjectMarker;
begin
  OMOpenDialog.InitialDir := ImagePath;
  OMOpenDialog.Filter := 'Raster Object File (*.rof)|*.rof';
  OMOpenDialog.FileName := ImagePath + ChangeFileExt(GRIPFileName,
    '.rof');
  if OMOpenDialog.Execute then
    OMFileName := OMOpenDialog.Filename;
{$I-}
  AssignFile(fLoad, OMFileName);
  Reset(fLoad);
  ObjectListCount := -1;
  while (not eof(fLoad)) do
  begin
    Read(fLoad, Incoming);
    inc(ObjectListCount);
    SetLength(ObjectMarkerArray, ObjectListCount + 1);
    ObjectMarkerArray[ObjectListCount] := Incoming;
    {ObjectMarker = record
ObjectType,X1,X2,Y1,Y2:Integer;
LineColor,FillColor:TColor;    }
  end;
  CloseFile(fLoad); {$I+}
  CurrentObjectMarker := 0;
  ShowData(CurrentObjectMarker);
  UpDown1.Position := CurrentObjectMarker;
end;

procedure TObjectMarkForm.SaveListBtnClick(Sender: TObject);
var
  I: Integer;
  fLoad: file of ObjectMarker;
  OutGoing: ObjectMarker;
begin {Save Training sites}
  OMSaveDialog1.InitialDir := ImagePath;
  OMSaveDialog1.FileName := ImagePath + ChangeFileExt(GRIPFileName,
    '.rof');
  OMSaveDialog1.Filter := 'Raster Object File (*.rof)|*.rof';
  if OMSaveDialog1.execute then begin
    OMFileName := OMSaveDialog1.Filename;
{$I-}
    AssignFile(fLoad, OMSaveDialog1.filename);
    Rewrite(fLoad);
    for I := 0 to ObjectListCount do
    begin
      OutGoing := ObjectMarkerArray[I];
      Write(fLoad, OutGoing);
    end;
    CloseFile(fLoad); {$I+}
  end;
end;


procedure TObjectMarkForm.OverlayListBtnClick(Sender: TObject);
begin
  PaintAll;
end;

procedure TObjectMarkForm.PaintAll;
var I: Integer;
begin
  with GRIPForm.Image1 do
  begin
    for I := 0 to ObjectListCount do
    begin {Point  Line  Circle Box}

      if ObjectMarkerArray[I].ObjectType = 0 then
      begin
        Canvas.Pen.Color := ObjectMarkerArray[I].LineColor;
        Canvas.Brush.Color := ObjectMarkerArray[I].FillColor;
        if ObjectMarkerArray[I].IsSolidFill then
          Canvas.Brush.Style := bsSolid
        else Canvas.Brush.Style := bsClear;
        Canvas.Pen.Color := ObjectMarkerArray[I].LineColor;
        Canvas.Ellipse(ObjectMarkerArray[I].X1 - 2,
          ObjectMarkerArray[I].Y1 - 2,
          ObjectMarkerArray[I].X2 + 2,
          ObjectMarkerArray[I].Y2 + 2);
      end else
        if ObjectMarkerArray[I].ObjectType = 1 then
        begin
          Canvas.Pen.Color := ObjectMarkerArray[I].LineColor;
          Canvas.MoveTo(ObjectMarkerArray[I].X1,
            ObjectMarkerArray[I].Y1);
          Canvas.LineTo(ObjectMarkerArray[I].X2,
            ObjectMarkerArray[I].Y2);
        end else
          if ObjectMarkerArray[I].ObjectType = 2 then
          begin
            Canvas.Brush.Color := ObjectMarkerArray[I].FillColor;
            if ObjectMarkerArray[I].IsSolidFill then
              Canvas.Brush.Style := bsSolid
            else Canvas.Brush.Style := bsClear;
            Canvas.Pen.Color := ObjectMarkerArray[I].LineColor;
            Canvas.Ellipse(ObjectMarkerArray[I].X1,
              ObjectMarkerArray[I].Y1,
              ObjectMarkerArray[I].X2,
              ObjectMarkerArray[I].Y2);
          end else
            if ObjectMarkerArray[I].ObjectType = 3 then
            begin
              Canvas.Brush.Color := ObjectMarkerArray[I].FillColor;
              if ObjectMarkerArray[I].IsSolidFill then
                Canvas.Brush.Style := bsSolid
              else Canvas.Brush.Style := bsClear;
              Canvas.Pen.Color := ObjectMarkerArray[I].LineColor;
              Canvas.Rectangle(ObjectMarkerArray[I].X1,
                ObjectMarkerArray[I].Y1,
                ObjectMarkerArray[I].X2,
                ObjectMarkerArray[I].Y2);
            end;
    end;
  end;
end;
{
ObjectRG.ItemIndex:=  ObjectMarkerArray[ObjectListCount].ObjectType;
LOX1Edit.Text:= IntToStr(  ObjectMarkerArray[ObjectListCount].X1);
LOX2Edit.Text:= IntToStr(  ObjectMarkerArray[ObjectListCount].X2);
LOY1Edit.Text:= IntToStr(  ObjectMarkerArray[ObjectListCount].Y1);
LOY2Edit.Text:= IntToStr(  ObjectMarkerArray[ObjectListCount].Y2);
LineColorPanel.Color:=  ObjectMarkerArray[ObjectListCount].LineColor;
FillPanel.Color:=  ObjectMarkerArray[ObjectListCount].FillColor;
FillCB.Checked:=  ObjectMarkerArray[ObjectListCount].IsSolidFill;
}


procedure TObjectMarkForm.ClearAllBtnClick(Sender: TObject);
begin
  UndoImage;
  ClearAllBtn.Enabled:=False;
end;

procedure TObjectMarkForm.MakeBtnClick(Sender: TObject);
begin
  MakeImage;
  ClearAllBtn.Enabled:=True;
end;



end.
